# -*- coding: utf-8 -*-

import unittest
import time

from flask import abort, Blueprint, Flask, url_for, redirect
from flask.ext.stats import Stats
from mock import Mock


class TestStats(unittest.TestCase):
    def _create_app(self, name):
        app = Flask(name)

        @app.route('/')
        def index():
            time.sleep(0.03)
            return ''

        @app.route('/other')
        def other():
            time.sleep(0.08)
            return ''

        @app.route('/redirect')
        def redir():
            return redirect(url_for('other'))

        @app.route('/slash/')
        def slash():
            return ''

        @app.route('/fail')
        def fail():
            abort(401)

        blueprint = Blueprint('balls', __name__)

        @blueprint.route('/')
        def blueprint_index():
            time.sleep(0.03)
            return ''

        @blueprint.route('/other')
        def blueprint_other():
            time.sleep(0.08)
            return ''

        @blueprint.route('/redirect')
        def blueprint_redir():
            return redirect(url_for('other'))

        @blueprint.route('/slash/')
        def blueprint_slash():
            return ''

        @blueprint.route('/fail')
        def blueprint_fail():
            abort(401)

        app.register_blueprint(blueprint, url_prefix='/blue')
        return app

    def _setup_mocks(self, stats_extension):
        pipeline_mock = Mock()
        pipeline_contextmanager_mock = Mock()
        pipeline_contextmanager_mock.__enter__ = Mock(
            return_value=pipeline_mock
        )
        pipeline_contextmanager_mock.__exit__ = Mock(return_value=False)

        stats_extension.pipeline = Mock()
        stats_extension.pipeline.return_value = pipeline_contextmanager_mock

        return pipeline_mock


class TestSetup(TestStats):
    def test_setup(self):
        app = self._create_app('hello')
        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/')

        self.assertTrue(pipeline.incr.called)
        pipeline.incr.assert_called_once_with(
            'index.http_200'
        )
        pipeline.timing.assert_called_once_with(
            'index', TimingValue(0.03 * 1000, 10)
        )

    def test_setup_single_application(self):
        app = self._create_app('hello')
        stats = Stats(app).client

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/')

        self.assertTrue(pipeline.incr.called)
        pipeline.incr.assert_called_once_with(
            'index.http_200'
        )
        pipeline.timing.assert_called_once_with(
            'index', TimingValue(0.03 * 1000, 10)
        )


class TimingValue(object):
    def __init__(self, expected_value, acceptable_variance):
        self.expected_value = expected_value
        self.acceptable_variance = acceptable_variance

    def __eq__(self, other):
        return (
            self.expected_value - self.acceptable_variance < other and
            self.expected_value + self.acceptable_variance > other
        )

    def __ne__(self, other):
        return not self.__eq__(other)



from flask.ext.stats import Stats

from . import TestStats, TimingValue


class TestAppRoutes(TestStats):
    def test_timer(self):
        app = self._create_app('hello')
        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/')

        self.assertTrue(pipeline.incr.called)
        pipeline.incr.assert_called_once_with(
            'index.http_200'
        )
        pipeline.timing.assert_called_once_with(
            'index', TimingValue(0.03 * 1000, 10)
        )


class TestAppNonExistentRoute(TestStats):
    def test_404(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/hello')

        self.assertFalse(pipeline.timing.called)

    def test_301_to_route_with_slash(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/slash')

        self.assertFalse(pipeline.timing.called)


class TestBlueprintRoutes(TestStats):
    def test_timer(self):
        app = self._create_app('hello')
        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/')

        self.assertTrue(pipeline.incr.called)
        pipeline.incr.assert_called_once_with(
            'index.http_200'
        )
        pipeline.timing.assert_called_once_with(
            'index', TimingValue(0.03 * 1000, 10)
        )


class TestBlueprintNonExistentRoute(TestStats):
    def test_404(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/hello')

        self.assertFalse(pipeline.timing.called)

    def test_301_to_route_with_slash(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/slash')

        self.assertFalse(pipeline.timing.called)



# -*- coding: utf-8 -*-

import time

import flask
from flask import request
from statsd import StatsClient

__author__ = 'Chris Trotman'
__email__ = 'chris@trotman.io'
__version__ = '0.1.0'


class _StatsClient(StatsClient):
    def flask_time_start(self):
        if not hasattr(flask.g, 'extensions'):
            flask.g.extensions = {}

        flask.g.extensions.setdefault('stats', {})
        flask.g.extensions['stats'][self] = time.time()

    def flask_time_end(self, response):
        end_time = time.time()
        start_time = flask.g.extensions['stats'][self]

        total_time = end_time - start_time
        time_in_ms = total_time * 1000

        endpoint = str(request.endpoint)
        with self.pipeline() as pipeline:
            if request.endpoint is not None:
                pipeline.timing(endpoint, time_in_ms)

            pipeline.incr(
                '%s.http_%s' % (endpoint, response.status_code)
            )

        return response


class Stats(object):
    def __init__(self, app=None):
        if app:
            self.app = app
            self.client = self.init_app(app)
        else:
            self.app = None
            self.client = None

    def init_app(self, app):
        """Inititialise the extension with the app object.

        :param app: Your application object
        """
        host = app.config.get('STATS_HOSTNAME', 'localhost')
        port = app.config.get('STATS_PORT', 8125)
        base_key = app.config.get('STATS_BASE_KEY', app.name)

        client = _StatsClient(
            host=host,
            port=port,
            prefix=base_key,
        )

        app.before_request(client.flask_time_start)
        app.after_request(client.flask_time_end)

        if not hasattr(app, 'extensions'):
            app.extensions = {}
        app.extensions.setdefault('stats', {})
        app.extensions['stats'][self] = client

        return client

    def __getattr__(self, name):
        return getattr(self.client, name, None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = open('README.rst').read()

requirements = [
    'Flask',
    'statsd',
]

test_requirements = open('requirements-testing.txt').readlines()

setup(
    name='Flask-Stats',
    version='1.0.1',
    description='A flask plugin to keep stats about your application',
    long_description=readme,
    author='Chris Trotman',
    author_email='chris@trotman.io',
    url='https://github.com/solarnz/Flask-Stats',
    packages=[
        'flask_stats',
    ],
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='Flask-Stats',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    test_suite='tests',
    tests_require=test_requirements
)



from flask.ext.stats import Stats

from . import TestStats


class TestAppRoutes(TestStats):
    def test_200(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/')

        pipeline.incr.assert_called_once_with(
            'index.http_200'
        )

    def test_302(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/redirect')

        pipeline.incr.assert_called_once_with(
            'redir.http_302'
        )

    def test_401(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/fail')

        pipeline.incr.assert_called_once_with(
            'fail.http_401'
        )


class TestAppNonExistentRoute(TestStats):
    def test_404(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/hello')

        pipeline.incr.assert_called_once_with(
            'None.http_404'
        )

    def test_301_to_route_with_slash(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/slash')

        pipeline.incr.assert_called_once_with(
            'None.http_301'
        )


class TestBlueprintRoutes(TestStats):
    def test_200(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/')

        pipeline.incr.assert_called_once_with(
            'balls.blueprint_index.http_200'
        )

    def test_302(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/redirect')

        pipeline.incr.assert_called_once_with(
            'balls.blueprint_redir.http_302'
        )

    def test_401(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/fail')

        pipeline.incr.assert_called_once_with(
            'balls.blueprint_fail.http_401'
        )


class TestBlueprintNonExistentRoute(TestStats):
    def test_404(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/hello')

        pipeline.incr.assert_called_once_with(
            'None.http_404'
        )

    def test_301_to_route_with_slash(self):
        app = self._create_app('hello')

        stats = Stats().init_app(app)

        pipeline = self._setup_mocks(stats)
        client = app.test_client()
        client.get('/blue/slash')

        pipeline.incr.assert_called_once_with(
            'None.http_301'
        )
